# -*- coding: utf-8 -*-

from plone import api


def fix_comments_1001(context):
    from plone.app.discussion.interfaces import IConversation

    brains = api.content.find(
        context=api.portal.get(),
        total_comments={"query": [1, 1000], "range": "min:max"},
    )
    for brain in brains:
        obj = brain.getObject()
        conversation = IConversation(obj, None)
        for thread in conversation.getThreads():
            comment = thread["comment"]
            if comment.author_name == comment.author_email:
                user = api.user.get(username=comment.author_username)
                infos = [user.getProperty("first_name"), user.getProperty("last_name")]
                comment.author_name = " ".join([i for i in infos if i])



# -*- coding: utf-8 -*-

from ideabox.policy.content.project import IProject
from plone.protect.auto import safeWrite
from zope.globalrequest import getRequest


def project_image_changed(obj, event):
    if hasattr(obj, "aq_parent") is False:
        return
    if IProject.providedBy(obj.aq_parent):
        obj.aq_parent.reindexObject(idxs=["project_picture"])


def project_added(obj, event):
    request = getRequest()
    safeWrite(obj, request)
    obj.allow_discussion = True



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import ideabox.policy


class IdeaboxPolicyLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.dexterity

        self.loadZCML(package=plone.app.dexterity)
        self.loadZCML(package=ideabox.policy)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "ideabox.policy:default")


IDEABOX_POLICY_FIXTURE = IdeaboxPolicyLayer()


IDEABOX_POLICY_INTEGRATION_TESTING = IntegrationTesting(
    bases=(IDEABOX_POLICY_FIXTURE,), name="IdeaboxPolicyLayer:IntegrationTesting"
)


IDEABOX_POLICY_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(IDEABOX_POLICY_FIXTURE,), name="IdeaboxPolicyLayer:FunctionalTesting"
)


IDEABOX_POLICY_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(IDEABOX_POLICY_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="IdeaboxPolicyLayer:AcceptanceTesting",
)



# encoding: utf-8

from Zope2.App import startup
from cioppino.twothumbs import rate
from plone import api
from plone.i18n.normalizer import urlnormalizer
from plone.namedfile.file import NamedBlobImage
from transaction import commit
from zope.component.hooks import setSite

import argparse
import csv
import os

from ideabox.policy.utils import token_type_recovery


def add_project(
    container,
    project_id,
    title,
    project_theme,
    project_body,
    image_source,
    project_author,
    project_like,
    project_unlike,
    project_status,
):
    img_extensions = ("jpg", "png", "gif")
    with api.env.adopt_user(username="admin"):
        project = api.content.create(
            type="Project",
            title=title,
            project_theme=project_theme,
            body=u"<br>".join(project_body.decode("utf8").splitlines()),
            container=container,
            original_author=project_author,
            original_id=project_id,
        )
        for ext in img_extensions:
            filename = "{0}.{1}".format(project_id, ext)
            img_path = os.path.join(image_source, filename)
            if os.path.exists(img_path):
                add_image_from_file(project, filename, image_source)
                break

    rate.setupAnnotations(project)
    if project_like:
        for x in range(0, int(project_like)):
            rate.loveIt(project, "userimportp" + str(x))
    if project_unlike:
        for x in range(0, int(project_unlike)):
            rate.hateIt(project, "userimportn" + str(x))

    with api.env.adopt_user(username="admin"):
        if project_status in ["selected", "rejected"]:
            api.content.transition(obj=project, transition="deposit")
            api.content.transition(obj=project, transition="to_be_analysed")
            api.content.transition(obj=project, transition="vote")
            api.content.transition(obj=project, transition="vote_analysis")
            if project_status == "selected":
                api.content.transition(obj=project, transition="accept")
            else:
                api.content.transition(obj=project, transition="reject")


def add_image_from_file(container, file_name, source):
    file_path = os.path.join(source, file_name)
    if not container.hasObject(file_name):
        # with deterity image
        named_blob_image = NamedBlobImage(
            data=open(file_path, "r").read(), filename=unicode(file_name)
        )
        image = api.content.create(
            type="Image",
            title=file_name,
            image=named_blob_image,
            container=container,
            language="fr",
        )
        image.setTitle(file_name)
        image.reindexObject()


def data_recovery(filename, image, container, status):
    csv_file = open(filename, "rb")
    reader = csv.reader(csv_file, delimiter=";")
    reader.next()
    for line in reader:
        project_id = line[0]
        project_title = line[2]
        project_theme = line[1]
        project_body = line[18]
        project_author = line[3]
        project_mail = line[4]
        project_like = line[15]
        project_unlike = line[16]

        token_type = token_type_recovery(project_theme)

        if len(project_author) < 3:
            project_author = urlnormalizer.normalize(
                project_mail[0:3].decode("utf8"), locale="fr"
            )
        else:
            project_author = urlnormalizer.normalize(
                project_author.decode("utf8"), locale="fr"
            )

        add_project(
            container,
            project_id,
            project_title,
            token_type,
            project_body,
            image,
            project_author,
            project_like,
            project_unlike,
            status,
        )
        print(project_title)


def follow_path(portal, path):
    element = portal
    for id in path.split("/"):
        if id not in element:
            raise ValueError("The id {0} does not exist".format(id))
        element = element[id]
    return element


def main(app):
    startup.startup()

    parser = argparse.ArgumentParser()
    parser.add_argument("-c")
    parser.add_argument("csv", help="csv file")
    parser.add_argument("img", help="localisation of pictures")
    parser.add_argument("name", help="Name of plone site")
    parser.add_argument("status", help="status of types")
    parser.add_argument("--path", dest="path", default="projets", help="creation path")
    args = parser.parse_args()

    setSite(app[args.name])
    portal = app[args.name]
    container = follow_path(portal, args.path)

    if args.csv and args.img:
        data_recovery(args.csv, args.img, container, args.status)
        commit()


if "app" in locals():
    main(app)  # NOQA



# -*- coding: utf-8 -*-

from DateTime import DateTime
from ZODB.PersistentMapping import PersistentMapping
from datetime import datetime
from plone import api
from plone.z3cform.layout import FormWrapper
from z3c.form import button
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import NO_VALUE
from zope import schema
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import implements
from zope.interface.interface import InterfaceClass

from ideabox.policy import _


class IWorkflowForm(Interface):
    """Marker interface for workflow history form"""


class WorkflowDataProvider(object):
    def get(self):
        return self.values.get(self.field.__name__, NO_VALUE)

    def __init__(self, context, request, form, field, widget):
        self.context = context
        self.request = request
        self.form = form
        self.field = field
        self.widget = widget

    @property
    def values(self):
        return {
            l.get("review_state"): l.get("time").asdatetime()
            for l in self.form.context.workflow_history.values()[0]
        }


class WorkflowHistoryForm(Form):
    implements(IWorkflowForm)
    ignoreContext = True

    _states = (
        "deposited",
        "project_analysis",
        "vote",
        "result_analysis",
        "selected",
        "rejected",
        "study_in_progress",
        "in_progress",
        "realized",
    )

    @property
    def workflow(self):
        return self.context.workflow_history.values()[0]

    def update(self):
        current_state = api.content.get_state(obj=self.context)
        states = self._states[: self._states.index(current_state) + 1]
        for idx, state in enumerate(states):
            self.fields += Fields(
                InterfaceClass(
                    "IWorkflowHistory{0}",
                    attrs={
                        state: schema.Date(
                            title=translate(state, domain="plone"), required=True
                        )
                        for l in self.workflow
                        if l.get("review_state") == state
                    },
                )
            )
        super(WorkflowHistoryForm, self).update()

    @button.buttonAndHandler(_(u"Save"), name="save")
    def handleApply(self, action):
        data, errors = self.extractData()
        workflow_history = self.context.workflow_history
        for line in workflow_history.values()[0]:
            state = line.get("review_state")
            if state in data:
                line["time"] = DateTime(
                    datetime.combine(data[state], datetime.min.time())
                )
        self.context.workflow_history = PersistentMapping(workflow_history.items())
        self.request.response.redirect(self.context.absolute_url())


class WorkflowEditView(FormWrapper):
    form = WorkflowHistoryForm
